@@ -2,7 +2,7 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 1996-2002, 2009, 2011, 2014 by                                */
+/*  Copyright (C) 1996-2021 by                                              */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*  grblit.c: Support for blitting of bitmaps with various depth.           */
@@ -10,7 +10,6 @@
 /****************************************************************************/
 
 #include "grblit.h"
-#include "grobjs.h"
 
 #define  GRAY8
 
@@ -142,14 +141,16 @@
   void  blit_mono_to_mono( grBlitter*  blit,
                            grColor     color )
   {
-    int    shift, left_clip, x, y;
+    unsigned int  shift;
+    int           left_clip, x, y;
+
     byte*  read;
     byte*  write;
 
     (void)color;   /* unused argument */
 
     left_clip = ( blit->xread > 0 );
-    shift     = ( blit->xwrite - blit->xread ) & 7;
+    shift     = (unsigned int)( blit->xwrite - blit->xread ) & 7;
 
     read  = blit->read  + (blit->xread >> 3);
     write = blit->write + (blit->xwrite >> 3);
@@ -195,19 +196,18 @@
         unsigned char*  _read  = read;
         unsigned char*  _write = write;
         unsigned int    old;
-        int             shift2 = (8-shift);
+        unsigned int    shift2 = (8-shift);
 
         if ( left_clip )
-          old = (*_read++) << shift2;
+          old = (unsigned int)(*_read++) << shift2;
         else
           old = 0;
 
         x = count;
         while ( x > 0 )
         {
-          unsigned char val;
+          unsigned int  val = *_read++;
 
-          val = *_read++;
           *_write++ |= (unsigned char)( (val >> shift) | old );
           old = val << shift2;
           x--;
@@ -235,31 +235,32 @@
   void  blit_mono_to_pal8( grBlitter*  blit,
                            grColor     color )
   {
-    int             x, y, shift;
-    unsigned char*  read;
-    unsigned char*  write;
-
-    read  = blit->read  + (blit->xread >> 3);
-    write = blit->write +  blit->xwrite;
-    shift = blit->xread & 7;
+    int             x, y;
+    unsigned char*  write = blit->write + blit->xwrite;
+    unsigned char*  read  = blit->read  + ( blit->xread >> 3 );
+    unsigned int    mask  = 0x80 >> ( blit->xread & 7 );
 
     y = blit->height;
     do
     {
-      unsigned char*  _read  = read;
       unsigned char*  _write = write;
-      unsigned long    val    = (*_read++ | 0x100) << shift;
+      unsigned char*  _read  = read;
+      unsigned int    _mask  = mask;
+      unsigned int     val   = *_read;
 
       x = blit->width;
       do
       {
-        if (val & 0x10000)
-          val = *_read++ | 0x100;
+        if ( !_mask )
+        {
+          val   = *++_read;
+          _mask = 0x80;
+        }
 
-        if ( val & 0x80 )
+        if ( val & _mask )
           *_write = (unsigned char)color.value;
 
-        val <<= 1;
+        _mask >>= 1;
         _write++;
 
       } while ( --x > 0 );
@@ -281,46 +282,37 @@
   void  blit_mono_to_pal4( grBlitter*  blit,
                            grColor     color )
   {
-    int             x, y, phase,shift;
-    unsigned char*  read;
-    unsigned char*  write;
-    unsigned int    col;
-
-
-    col   = color.value & 15;
-    read  = blit->read  + (blit->xread >> 3);
-    write = blit->write + (blit->xwrite >> 1);
-
-    /* now begin blit */
-    shift = blit->xread & 7;
-    phase = blit->xwrite & 1;
+    int             x, y;
+    unsigned char*  write = blit->write + ( blit->xwrite >> 1 );
+    unsigned int    phase = blit->xwrite & 1 ? 0x0F : 0xF0;
+    unsigned char*  read  = blit->read  + ( blit->xread >> 3 );
+    unsigned int    mask  = 0x80 >> ( blit->xread & 7 );
+    unsigned int    col = color.value | ( color.value << 4 );
 
     y = blit->height;
     do
     {
-      unsigned char*  _read  = read;
       unsigned char*  _write = write;
-      int             _phase = phase;
-      unsigned long    val    = (*_read++ | 0x100) << shift;
+      unsigned int    _phase = phase;
+      unsigned char*  _read  = read;
+      unsigned int    _mask  = mask;
+      unsigned int     val   = *_read;
 
       x = blit->width;
       do
       {
-        if (val & 0x10000)
-          val = *_read++ | 0x100;
-
-        if ( val & 0x80 )
+        if ( !_mask )
         {
-          if ( _phase )
-            *_write = (unsigned char)((*_write & 0xF0) | col);
-          else
-            *_write = (unsigned char)((*_write & 0x0F) | (col << 4));
+          val   = *++_read;
+          _mask = 0x80;
         }
 
-        val <<= 1;
+        if ( val & _mask )
+          *_write = (unsigned char)( (col & _phase) | (*_write & ~_phase) );
 
-        _write += _phase;
-        _phase ^= 1;
+        _mask >>= 1;
+        _write += _phase & 1;
+        _phase  = ~_phase;
         x--;
       } while ( x > 0 );
 
@@ -341,32 +333,33 @@
   void  blit_mono_to_rgb16( grBlitter*  blit,
                             grColor     color )
   {
-    int              x, y,shift;
-    unsigned char*   read;
-    unsigned char*   write;
-
-    read  = blit->read + (blit->xread >> 3);
-    write = blit->write + blit->xwrite*2;
-    shift = blit->xread & 7;
+    int              x, y;
+    unsigned char*   write = blit->write + blit->xwrite * 2;
+    unsigned char*   read  = blit->read  + ( blit->xread >> 3 );
+    unsigned int     mask  = 0x80 >> ( blit->xread & 7 );
 
     y = blit->height;
     do
     {
+      unsigned short* _write = (unsigned short*)write;
       unsigned char*  _read  = read;
-      unsigned char*  _write = write;
-      unsigned long    val    = (*_read++ | 0x100) << shift;
+      unsigned int    _mask  = mask;
+      unsigned int     val   = *_read;
 
       x = blit->width;
       do
       {
-        if (val & 0x10000)
-          val = *_read++ | 0x100;
+        if ( !_mask )
+        {
+          val   = *++_read;
+          _mask = 0x80;
+        }
 
-        if ( val & 0x80 )
-          *(short*)_write = (short)color.value;
+        if ( val & _mask )
+          *_write = (unsigned short)color.value;
 
-        val   <<= 1;
-        _write +=2;
+        _mask >>= 1;
+        _write++;
         x--;
       } while ( x > 0 );
 
@@ -387,35 +380,36 @@
   void  blit_mono_to_rgb24( grBlitter*  blit,
                             grColor     color )
   {
-    int             x, y, shift;
-    unsigned char*  read;
-    unsigned char*  write;
-
-    read  = blit->read  + (blit->xread >> 3);
-    write = blit->write + blit->xwrite*3;
-    shift = blit->xread & 7;
+    int             x, y;
+    unsigned char*  write = blit->write + blit->xwrite * 3;
+    unsigned char*  read  = blit->read  + ( blit->xread >> 3 );
+    unsigned int    mask  = 0x80 >> ( blit->xread & 7 );
 
     y = blit->height;
     do
     {
-      unsigned char*  _read  = read;
       unsigned char*  _write = write;
-      unsigned long    val   = (*_read++ | 0x100) << shift;
+      unsigned char*  _read  = read;
+      unsigned int    _mask  = mask;
+      unsigned int     val   = *_read;
 
       x = blit->width;
       do
       {
-        if (val & 0x10000)
-          val = *_read++ | 0x100;
+        if ( !_mask )
+        {
+          val   = *++_read;
+          _mask = 0x80;
+        }
 
-        if ( val & 0x80 )
+        if ( val & _mask )
         {
           _write[0] = color.chroma[0];
           _write[1] = color.chroma[1];
           _write[2] = color.chroma[2];
         }
 
-        val   <<= 1;
+        _mask >>= 1;
         _write += 3;
         x--;
       } while ( x > 0 );
@@ -437,42 +431,33 @@
   void  blit_mono_to_rgb32( grBlitter*  blit,
                             grColor     color )
   {
-    int             x, y,shift;
-    unsigned char*  read;
-    unsigned char*  write;
-
-    read  = blit->read  + ( blit->xread >> 3 );
-    write = blit->write + blit->xwrite*4;
-    shift = blit->xread & 7;
+    int             x, y;
+    unsigned char*  write = blit->write + blit->xwrite * 4;
+    unsigned char*  read  = blit->read  + ( blit->xread >> 3 );
+    unsigned int    mask  = 0x80 >> ( blit->xread & 7 );
 
     y = blit->height;
     do
     {
+      uint32_t*       _write = (uint32_t*)write;
       unsigned char*  _read  = read;
-      unsigned char*  _write = write;
-      unsigned long   val    = ( *_read++ | 0x100L ) << shift;
+      unsigned int    _mask  = mask;
+      unsigned int     val   = *_read;
 
       x = blit->width;
       do
       {
-        if ( val & 0x10000 )
-          val = *_read++ | 0x100L;
-
-        if ( val & 0x80 )
+        if ( !_mask )
         {
-          /* this could be greatly optimized as                         */
-          /*                                                            */
-          /*   *(long*)_write = color.value                             */
-          /*                                                            */
-          /* but it wouldn't work on 64-bits systems... stupid C types! */
-          _write[0] = color.chroma[0];
-          _write[1] = color.chroma[1];
-          _write[2] = color.chroma[2];
-          _write[3] = color.chroma[3];
+          val   = *++_read;
+          _mask = 0x80;
         }
 
-        val   <<= 1;
-        _write += 4;
+        if ( val & _mask )
+          *_write = color.value;
+
+        _mask >>= 1;
+        _write++;
         x--;
 
       } while ( x > 0 );
@@ -540,8 +525,8 @@
   grSaturation*  gr_last_saturation = gr_saturations;
 
 
-  extern
-  const byte*  grGetSaturation( int  num_grays )
+  static const byte*
+  grGetSaturation( int  num_grays )
   {
     /* first of all, scan the current saturations table */
     grSaturation*  sat   = gr_saturations;
@@ -550,7 +535,7 @@
     if ( num_grays < 2 )
     {
       grError = gr_err_bad_argument;
-      return 0;
+      return NULL;
     }
 
     for ( ; sat < limit; sat++ )
@@ -565,27 +550,28 @@
     /* not found, simply create a new entry if there is room */
     if (gr_num_saturations < GR_MAX_SATURATIONS)
     {
-      int          i;
-      const byte*  table;
+      int    i;
+      byte*  table;
 
-      table = (const byte*)grAlloc( (3*num_grays-1)*sizeof(byte) );
-      if (!table) return 0;
+      table = grAlloc( (size_t)( 3 * num_grays - 1 ) * sizeof ( byte ) );
+      if (!table)
+        return NULL;
 
       sat->count = num_grays;
       sat->table = table;
 
       for ( i = 0; i < num_grays; i++, table++ )
-        *(unsigned char*)table = (unsigned char)i;
+        *table = (byte)i;
 
       for ( i = 2*num_grays-1; i > 0; i--, table++ )
-        *(unsigned char*)table = (unsigned char)(num_grays-1);
+        *table = (byte)(num_grays-1);
 
       gr_num_saturations++;
       gr_last_saturation = sat;
-      return sat->table;
+      return table;
     }
     grError = gr_err_saturation_overflow;
-    return 0;
+    return NULL;
   }
 
 
@@ -635,9 +621,9 @@
   grConversion*  gr_last_conversion = gr_conversions;
 
 
-  extern
-  const byte*  grGetConversion( int  target_grays,
-                                int  source_grays )
+  static const byte*
+  grGetConversion( int  target_grays,
+                   int  source_grays )
   {
     grConversion*  conv  = gr_conversions;
     grConversion*  limit = conv + gr_num_conversions;
@@ -645,7 +631,7 @@
     if ( target_grays < 2 || source_grays < 2 )
     {
       grError = gr_err_bad_argument;
-      return 0;
+      return NULL;
     }
 
     /* otherwise, scan table */
@@ -662,27 +648,26 @@
     /* not found, add a new conversion to the table */
     if (gr_num_conversions < GR_MAX_CONVERSIONS)
     {
-      const byte*  table;
-      int          n;
+      byte*  table;
+      int    n;
 
-      table = (const byte*)grAlloc( source_grays*sizeof(byte) );
+      table = grAlloc( (size_t)source_grays * sizeof ( byte ) );
       if (!table)
-        return 0;
+        return NULL;
 
       conv->target_grays = target_grays;
       conv->source_grays = source_grays;
       conv->table        = table;
 
       for ( n = 0; n < source_grays; n++ )
-        ((unsigned char*)table)[n] = (unsigned char)(n*(target_grays-1) /
-                                         (source_grays-1));
+        table[n] = (byte)(n*(target_grays-1) / (source_grays-1));
 
       gr_num_conversions++;
       gr_last_conversion = conv;
       return table;
     }
     grError = gr_err_conversion_overflow;
-    return 0;
+    return NULL;
   }
 
 
@@ -702,7 +687,10 @@
     int             y;
     unsigned char*  read;
     unsigned char*  write;
-
+#ifdef GR_CONFIG_GRAY_SKIP_WHITE
+    int  max = blit->source.grays - 1;
+    int  max2 = blit-target.grays - 1;
+#endif
 
     read  = blit->read  + blit->xread;
     write = blit->write + blit->xwrite;
@@ -717,12 +705,12 @@
       while (x > 0)
       {
 #ifdef GR_CONFIG_GRAY_SKIP_WHITE
-        unsigned char val = *_read;
+        int  val = *_read;
 
         if (val)
         {
           if (val == max)
-            *_write = max2;
+            *_write = (byte)max2;
           else
             *_write = saturation[ (int)*_write + conversion[ *_read ] ];
         }
@@ -756,10 +744,7 @@
     unsigned char*  read;
     unsigned char*  write;
 #ifdef GR_CONFIG_GRAY_SKIP_WHITE
-    unsigned char   max;
-
-
-    max = (unsigned char)( blit->source.grays - 1 );
+    int  max = blit->source.grays - 1;
 #endif
 
     read  = blit->read  + blit->xread;
@@ -775,7 +760,7 @@
       while (x > 0)
       {
 #ifdef GR_CONFIG_GRAY_SKIP_WHITE
-        unsigned char val = *_read;
+        int  val = *_read;
 
         if (val)
         {
@@ -801,20 +786,21 @@
 
 
 
-#define compose_pixel_full( a, b, n0, n1, n2, max )          \
-  {                                                          \
-    int  d, half = max >> 1;                                 \
-                                                             \
-                                                             \
-    d = (int)b.chroma[0] - a.chroma[0];                      \
-    a.chroma[0] += (unsigned char)((n0*d + half)/max);       \
-                                                             \
-    d = (int)b.chroma[1] - a.chroma[1];                      \
-    a.chroma[1] += (unsigned char)((n1*d + half)/max);       \
-                                                             \
-    d = (int)b.chroma[2] - a.chroma[2];                      \
-    a.chroma[2] += (unsigned char)((n2*d + half)/max);       \
-  }
+#define compose_pixel_full( a, b, n0, n1, n2, max )    \
+  do                                                   \
+  {                                                    \
+    int  d, half = max >> 1;                           \
+                                                       \
+                                                       \
+    d = (int)b.chroma[0] - a.chroma[0];                \
+    a.chroma[0] += (unsigned char)((n0*d + half)/max); \
+                                                       \
+    d = (int)b.chroma[1] - a.chroma[1];                \
+    a.chroma[1] += (unsigned char)((n1*d + half)/max); \
+                                                       \
+    d = (int)b.chroma[2] - a.chroma[2];                \
+    a.chroma[2] += (unsigned char)((n2*d + half)/max); \
+  } while ( 0 )
 
 #define compose_pixel( a, b, n, max )  \
     compose_pixel_full( a, b, n, n, n, max )
@@ -823,25 +809,25 @@
 #define extract555( pixel, color )                           \
    color.chroma[0] = (unsigned char)((pixel >> 10) & 0x1F);  \
    color.chroma[1] = (unsigned char)((pixel >>  5) & 0x1F);  \
-   color.chroma[2] = (unsigned char)((pixel      ) & 0x1F);
+   color.chroma[2] = (unsigned char)((pixel      ) & 0x1F)
 
 
 #define extract565( pixel, color )                           \
    color.chroma[0] = (unsigned char)((pixel >> 11) & 0x1F);  \
    color.chroma[1] = (unsigned char)((pixel >>  5) & 0x3F);  \
-   color.chroma[2] = (unsigned char)((pixel      ) & 0x1F);
+   color.chroma[2] = (unsigned char)((pixel      ) & 0x1F)
 
 
 #define inject555( color )                          \
-   ( ( (unsigned short)color.chroma[0] << 10 ) |    \
-     ( (unsigned short)color.chroma[1] <<  5 ) |    \
-       color.chroma[2]                         )
+   (unsigned short)( ( color.chroma[0] << 10 ) |    \
+                     ( color.chroma[1] <<  5 ) |    \
+                       color.chroma[2]           )
 
 
 #define inject565( color )                          \
-   ( ( (unsigned short)color.chroma[0] << 11 ) |    \
-     ( (unsigned short)color.chroma[1] <<  5 ) |    \
-       color.chroma[2]                         )
+   (unsigned short)( ( color.chroma[0] << 11 ) |    \
+                     ( color.chroma[1] <<  5 ) |    \
+                       color.chroma[2]           )
 
 
 /**************************************************************************/
@@ -856,63 +842,51 @@
                            grColor     color )
   {
     int             y;
-    int             sr = (color.chroma[0] << 8) & 0x7C00;
-    int             sg = (color.chroma[1] << 2) & 0x03E0;
-    int             sb = (color.chroma[2]     ) & 0x001F;
+    int             sr = color.value & 0x7C00;
+    int             sg = color.value & 0x03E0;
+    int             sb = color.value & 0x001F;
     unsigned char*  read;
     unsigned char*  write;
-    long            color2;
 
     read   = blit->read  + blit->xread;
     write  = blit->write + 2*blit->xwrite;
 
-    color2 = color.value;
-    extract565( color2, color );
-
     y = blit->height;
     do
     {
-      unsigned char*  _read  = read;
-      unsigned char*  _write = write;
-      int             x      = blit->width;
+      unsigned char*   _read  = read;
+      unsigned short*  _write = (unsigned short*)write;
+      int              x      = blit->width;
 
       while (x > 0)
       {
-        unsigned char    val;
+        unsigned char  val = *_read;
 
-        val = *_read;
         if (val)
         {
-          unsigned short* pixel = (unsigned short*)_write;
-
           if (val >= 254 )
-          {
-            pixel[0] = (short)color2;
-          }
+            *_write = (unsigned short)( sr | sg | sb );
           else if ( val >= 2 )
           {
             /* compose gray value */
-            int   pix = (int)*pixel;
+            int   pix = (int)*_write;
             int   dr  = pix & 0x7C00;
             int   dg  = pix & 0x03E0;
             int   db  = pix & 0x001F;
 
-            dr  = pix & 0x7C00;
             dr += ((sr-dr)*val) >> 8;
-            dr &= 0xF800;
+            dr &= 0x7C00;
 
-            dg  = pix & 0x03E0;
             dg += ((sg-dg)*val) >> 8;
-            dg &= 0x7E0;
+            dg &= 0x03E0;
 
-            db  = pix & 0x001F;
             db += ((sb-db)*val) >> 8;
-            db += 0x001F;
+            db &= 0x001F;
 
-            *pixel = (short)( dr | dg | db );
+            *_write = (unsigned short)( dr | dg | db );
           }
         }
-        _write +=2;
+        _write ++;
         _read  ++;
         x--;
       }
@@ -934,48 +908,43 @@
     int             y;
     unsigned char*  read;
     unsigned char*  write;
-    long            color2;
 
     read   = blit->read  + blit->xread;
     write  = blit->write + 2*blit->xwrite;
 
-    /* convert color to R:G:B triplet */
-    color2 = color.value;
-    extract555( color2, color );
+    /* scale down R:G:B triplet */
+    color.chroma[0] >>= 3;
+    color.chroma[1] >>= 3;
+    color.chroma[2] >>= 3;
 
     y = blit->height;
     do
     {
-      unsigned char*  _read  = read;
-      unsigned char*  _write = write;
-      int             x      = blit->width;
+      unsigned char*   _read  = read;
+      unsigned short*  _write = (unsigned short*)write;
+      int              x      = blit->width;
 
       while (x > 0)
       {
-        unsigned char   val;
+        int  val = *_read;
 
-        val = *_read;
         if (val)
         {
-          unsigned short* pixel = (unsigned short*)_write;
-
           if (val == max)
-          {
-            pixel[0] = (short)color2;
-          }
+            *_write = inject555( color );
           else
           {
             /* compose gray value */
-            unsigned short  pix16 = *pixel;
+            unsigned short  pix16 = *_write;
             grColor         pix;
 
             extract555( pix16, pix );
 
             compose_pixel( pix, color, val, max );
-            *pixel = (unsigned short)(inject555(pix));
+            *_write = inject555( pix );
           }
         }
-        _write += 2;
+        _write ++;
         _read  ++;
         x--;
       }
@@ -1000,63 +969,51 @@
                            grColor     color )
   {
     int             y;
-    int             sr = (color.chroma[0] << 8) & 0xF800;
-    int             sg = (color.chroma[1] << 2) & 0x07E0;
-    int             sb = (color.chroma[2]     ) & 0x001F;
+    int             sr = color.value & 0xF800;
+    int             sg = color.value & 0x07E0;
+    int             sb = color.value & 0x001F;
     unsigned char*  read;
     unsigned char*  write;
-    long            color2;
 
     read   = blit->read  + blit->xread;
     write  = blit->write + 2*blit->xwrite;
 
-    color2 = color.value;
-    extract565( color2, color );
-
     y = blit->height;
     do
     {
-      unsigned char*  _read  = read;
-      unsigned char*  _write = write;
-      int             x      = blit->width;
+      unsigned char*   _read  = read;
+      unsigned short*  _write = (unsigned short*)write;
+      int              x      = blit->width;
 
       while (x > 0)
       {
-        unsigned char    val;
+        unsigned char  val = *_read;
 
-        val = *_read;
         if (val)
         {
-          unsigned short* pixel = (unsigned short*)_write;
-
           if (val >= 254 )
-          {
-            pixel[0] = (short)color2;
-          }
+            *_write = (unsigned short)( sr | sg | sb );
           else if ( val >= 2 )
           {
             /* compose gray value */
-            int   pix = (int)*pixel;
+            int   pix = (int)*_write;
             int   dr  = pix & 0xF800;
             int   dg  = pix & 0x07E0;
             int   db  = pix & 0x001F;
 
-            dr  = pix & 0xF800;
             dr += ((sr-dr)*val) >> 8;
             dr &= 0xF800;
 
-            dg  = pix & 0x07E0;
             dg += ((sg-dg)*val) >> 8;
-            dg &= 0x7E0;
+            dg &= 0x07E0;
 
-            db  = pix & 0x001F;
             db += ((sb-db)*val) >> 8;
-            db += 0x001F;
+            db &= 0x001F;
 
-            *pixel = (short)( dr | dg | db );
+            *_write = (unsigned short)( dr | dg | db );
           }
         }
-        _write +=2;
+        _write ++;
         _read  ++;
         x--;
       }
@@ -1077,47 +1034,43 @@
     int             y;
     unsigned char*  read;
     unsigned char*  write;
-    long            color2;
 
     read   = blit->read  + blit->xread;
     write  = blit->write + 2*blit->xwrite;
 
-    color2 = color.value;
-    extract565( color2, color );
+    /* scale down R:G:B triplet */
+    color.chroma[0] >>= 3;
+    color.chroma[1] >>= 2;
+    color.chroma[2] >>= 3;
 
     y = blit->height;
     do
     {
-      unsigned char*  _read  = read;
-      unsigned char*  _write = write;
-      int             x      = blit->width;
+      unsigned char*   _read  = read;
+      unsigned short*  _write = (unsigned short*)write;
+      int              x      = blit->width;
 
       while (x > 0)
       {
-        unsigned char    val;
+        int  val = *_read;
 
-        val = *_read;
         if (val)
         {
-          unsigned short* pixel = (unsigned short*)_write;
-
           if (val == max)
-          {
-            pixel[0] = (short)color2;
-          }
+            *_write = inject565( color );
           else
           {
             /* compose gray value */
-            unsigned short  pix16 = *pixel;
+            unsigned short  pix16 = *_write;
             grColor         pix;
 
             extract565( pix16, pix );
 
             compose_pixel( pix, color, val, max );
-            *pixel = (short)inject565( pix );
+            *_write = inject565( pix );
           }
         }
-        _write +=2;
+        _write ++;
         _read  ++;
         x--;
       }
@@ -1160,9 +1113,8 @@
 
       while (x > 0)
       {
-        unsigned char    val;
+        unsigned char  val = *_read;
 
-        val = *_read;
         if (val)
         {
           if (val >= 254)
@@ -1182,7 +1134,7 @@
             db += ((sb-db)*val) >> 8;
 
             _write[0] = (unsigned char)dr;
-            _write[1] = (unsigned char)dg,
+            _write[1] = (unsigned char)dg;
             _write[2] = (unsigned char)db;
           }
         }
@@ -1221,9 +1173,8 @@
 
       while (x > 0)
       {
-        unsigned char    val;
+        int  val = *_read;
 
-        val = *_read;
         if (val)
         {
           if (val == max)
@@ -1288,9 +1239,8 @@
 
       while (x > 0)
       {
-        unsigned char  val;
+        int  val = *_read;
 
-        val = *_read;
         if (val)
         {
           if (val == max)
@@ -1367,11 +1317,9 @@
 
       while (x > 0)
       {
-        int    val0, val1, val2;
-
-        val0 = _read[0];
-        val1 = _read[1];
-        val2 = _read[2];
+        int  val0 = _read[0];
+        int  val1 = _read[1];
+        int  val2 = _read[2];
 
         if ( val0 | val1 | val2 )
         {
@@ -1397,9 +1345,9 @@
             db  = _write[2];
             db += (sb-db)*val2 >> 8;
 
-            _write[0] = dr;
-            _write[1] = dg;
-            _write[2] = db;
+            _write[0] = (unsigned char)dr;
+            _write[1] = (unsigned char)dg;
+            _write[2] = (unsigned char)db;
           }
         }
         _write += 3;
@@ -1436,11 +1384,9 @@
 
       while (x > 0)
       {
-        int    val0, val1, val2;
-
-        val0 = _read[0];
-        val1 = _read[1];
-        val2 = _read[2];
+        int  val0 = _read[0];
+        int  val1 = _read[1];
+        int  val2 = _read[2];
 
         if ( val0 | val1 | val2 )
         {
@@ -1505,11 +1451,9 @@
 
       while (x > 0)
       {
-        int    val0, val1, val2;
-
-        val0 = _read[2];
-        val1 = _read[1];
-        val2 = _read[0];
+        int  val0 = _read[2];
+        int  val1 = _read[1];
+        int  val2 = _read[0];
 
         if ( val0 | val1 | val2 )
         {
@@ -1532,12 +1476,12 @@
             dg  = _write[1];
             dg += (sg-dg)*val1 >> 8;
 
-            db  = _write[1];
+            db  = _write[2];
             db += (sb-db)*val2 >> 8;
 
-            _write[0] = dr;
-            _write[1] = dg;
-            _write[2] = db;
+            _write[0] = (unsigned char)dr;
+            _write[1] = (unsigned char)dg;
+            _write[2] = (unsigned char)db;
           }
         }
         _write += 3;
@@ -1574,11 +1518,9 @@
 
       while (x > 0)
       {
-        int    val0, val1, val2;
-
-        val0 = _read[2];
-        val1 = _read[1];
-        val2 = _read[0];
+        int  val0 = _read[2];
+        int  val1 = _read[1];
+        int  val2 = _read[0];
 
         if ( val0 | val1 | val2 )
         {
@@ -1648,11 +1590,9 @@
 
       while (x > 0)
       {
-        unsigned char    val0, val1, val2;
-
-        val0 = _read[0*line];
-        val1 = _read[1*line];
-        val2 = _read[2*line];
+        int  val0 = _read[0*line];
+        int  val1 = _read[1*line];
+        int  val2 = _read[2*line];
 
         if ( val0 | val1 | val2 )
         {
@@ -1716,11 +1656,9 @@
 
       while (x > 0)
       {
-        unsigned char    val0, val1, val2;
-
-        val0 = _read[2*line];
-        val1 = _read[1*line];
-        val2 = _read[0*line];
+        int  val0 = _read[2*line];
+        int  val1 = _read[1*line];
+        int  val2 = _read[0*line];
 
         if ( val0 | val1 | val2 )
         {
@@ -1761,25 +1699,6 @@
   }
 
 
- /**********************************************************************
-  *
-  * <Function>
-  *    grBlitGlyphBitmap
-  *
-  * <Description>
-  *    writes a given glyph bitmap to a target surface.
-  *
-  * <Input>
-  *    surface :: handle to target surface
-  *    x       :: position of left-most pixel of glyph image in surface
-  *    y       :: position of top-most pixel of glyph image in surface
-  *    bitmap  :: source glyph image
-  *
-  * <Return>
-  *   Error code. 0 means success
-  *
-  **********************************************************************/
-
   typedef  void (*grColorGlyphBlitter)( grBlitter*  blit,
                                         grColor     color,
                                         int         max_gray );
@@ -1818,16 +1737,6 @@
 #endif
 
 
-#include "gblblit.h"
-
-  static double    gr_glyph_gamma = 1.2;
-
-  void  grSetGlyphGamma( double  gamma )
-  {
-    gr_glyph_gamma = gamma;
-  }
-
-
   int
   grBlitGlyphToBitmap( grBitmap*  target,
                        grBitmap*  glyph,
@@ -1852,88 +1761,6 @@
       return 0;
     }
 
-   /* short cut to alpha blender for certain glyph types
-    */
-    {
-      GBlenderSourceFormat  src_format;
-      GBlenderTargetFormat  dst_format;
-      int                   width, height;
-      GBlenderBlitRec       gblit[1];
-      GBlenderPixel         gcolor;
-      static GBlenderRec    gblender[1];
-      static double         gblender_gamma = -100.0;
-
-      if ( glyph->grays != 256 )
-        goto DefaultBlit;
-
-      switch ( glyph->mode )
-      {
-      case gr_pixel_mode_gray:  src_format = GBLENDER_SOURCE_GRAY8; break;
-      case gr_pixel_mode_lcd:   src_format = GBLENDER_SOURCE_HRGB;  break;
-      case gr_pixel_mode_lcdv:  src_format = GBLENDER_SOURCE_VRGB;  break;
-      case gr_pixel_mode_lcd2:  src_format = GBLENDER_SOURCE_HBGR;  break;
-      case gr_pixel_mode_lcdv2: src_format = GBLENDER_SOURCE_VBGR;  break;
-      case gr_pixel_mode_bgra:  src_format = GBLENDER_SOURCE_BGRA;  break;
-
-      default:
-          goto DefaultBlit;
-      }
-
-      width  = glyph->width;
-      height = glyph->rows;
-
-      if ( glyph->mode == gr_pixel_mode_lcd  ||
-           glyph->mode == gr_pixel_mode_lcd2 )
-        width /= 3;
-
-      if ( glyph->mode == gr_pixel_mode_lcdv  ||
-           glyph->mode == gr_pixel_mode_lcdv2 )
-        height /= 3;
-
-      switch ( target->mode )
-      {
-      case gr_pixel_mode_rgb32: dst_format  = GBLENDER_TARGET_RGB32; break;
-      case gr_pixel_mode_rgb24: dst_format  = GBLENDER_TARGET_RGB24; break;
-      case gr_pixel_mode_rgb565: dst_format = GBLENDER_TARGET_RGB565; break;
-      default:
-          goto DefaultBlit;
-      }
-
-     /* initialize blender when needed, i.e. when gamma changes
-      */
-      if ( gblender_gamma != gr_glyph_gamma  )
-      {
-        gblender_gamma = gr_glyph_gamma;
-        gblender_init( gblender, gblender_gamma );
-      }
-
-      if ( gblender_blit_init( gblit, gblender,
-                               x, y,
-                               src_format,
-                               glyph->buffer,
-                               glyph->pitch,
-                               width,
-                               height,
-                               dst_format,
-                               target->buffer,
-                               target->pitch,
-                               target->width,
-                               target->rows ) < 0 )
-      {
-        /* nothing to do */
-        return 0;
-      }
-
-      gcolor = ((GBlenderPixel)color.chroma[0] << 16) |
-               ((GBlenderPixel)color.chroma[1] << 8 ) |
-               ((GBlenderPixel)color.chroma[2]      ) ;
-
-      gblender_blit_run( gblit, gcolor );
-      return 1;
-    }
-
-  DefaultBlit:
-
     /* set up blitter and compute clipping.  Return immediately if needed */
     blit.source = *glyph;
     blit.target = *target;
@@ -2035,6 +1862,7 @@
         blit_lcdv_to_24( &blit, color, glyph->grays-1 );
         break;
       }
+      /* fall through */
 
     case gr_pixel_mode_lcd2:
       if ( mode == gr_pixel_mode_rgb24 )
@@ -2050,12 +1878,12 @@
       break;
 
     case gr_pixel_mode_lcdv2:
-     if ( mode == gr_pixel_mode_rgb24 )
-     {
-       if ( glyph->grays > 1 )
-         blit_lcdv2_to_24( &blit, color, glyph->grays-1 );
-     }
-     break;
+      if ( mode == gr_pixel_mode_rgb24 )
+      {
+        if ( glyph->grays > 1 )
+          blit_lcdv2_to_24( &blit, color, glyph->grays-1 );
+      }
+      break;
 
     default:
       /* we don't support the blitting of bitmaps of the following  */
